# NOTE!: this code is mostly broken and is a work in progress

import coremltools.converters.keras
import keras
import sys


if len(sys.argv) != 2 or not (sys.argv[1].endswith('.h5') or sys.argv[1].endswith('.hdf5')):
    print('usage: keras_to_coreml.py <model file>')
    sys.exit(1)


model_filename = sys.argv[1]
model = keras.models.load_model(model_filename, custom_objects={
    # for mobilenet import, doesn't affect other model types
    'relu6': keras.applications.mobilenet.relu6,
    'DepthwiseConv2D': keras.applications.mobilenet.DepthwiseConv2D
})

# cut out to_multi_gpu stuff (this might break some models which don't use to_multi_gpu, just comment it out if so)
stripped_model = next((l for l in model.layers if isinstance(l, keras.engine.training.Model)), None)
if stripped_model:
    model_filename = model_filename + '-stripped.hdf5'
    stripped_model.save(model_filename)
    model = stripped_model


# coreml doesn't support Keras SeparableConv2D layers, so we just treat them as Conv2D layers instead
# this will not work. instead, we will need to write a SeparableConv2D implementation in terms of Conv2D (here's TensorFlow's version: https://github.com/tensorflow/tensorflow/commit/975094c8382ee15dad434278804cb096f9bac84f)
# code below uses private APIs, it will probably break (and doesn't work) -- hopefully coreml ships official support soon

# import coremltools.converters.keras._keras2_converter, coremltools.converters.keras._layers2
# coremltools.converters.keras._keras2_converter._KERAS_LAYER_REGISTRY[keras.layers.convolutional.SeparableConv2D] = coremltools.converters.keras._layers2.convert_convolution


coreml_model = coremltools.converters.keras.convert(
    model, input_names=['image'], image_input_names=['image'], output_names=['class_prob'],
    class_labels=['benign', 'malignant'], predicted_feature_name='lesion_type'
)

# model metadata
coreml_model.author = 'Triage'
coreml_model.short_description = 'Predicts benign vs. malignant lesions.'

# feature descriptions
coreml_model.input_description['image'] = 'Image'
coreml_model.output_description['class_prob'] = 'Probabilities of, in order, [benign, malignant]'

# save
coreml_model.save('BenignMalignant.mlmodel')



import os
import keras
import tensorflow
from keras_model_specs.models.custom_layers import Scale
try:
    # for newer versions of Keras
    import keras_applications
except ImportError:
    # for older versions of Keras
    import keras.applications as keras_applications


class KerasToTensorflow(object):

    @staticmethod
    def load_keras_model(model_path):
        custom_objects = {
            # just in case you have Lambda layers which implicitly 'import tensorflow as tf'
            # (happens to be the case for some of our internal code)
            'tf': tensorflow,

            # needed for Resnet152 support
            'Scale': Scale
        }
        try:
            # for mobilenet import, doesn't affect other model types
            custom_objects['relu6'] = keras_applications.mobilenet.layers.ReLU(6, name='relu6')
            custom_objects['DepthwiseConv2D'] = keras_applications.mobilenet.DepthwiseConv2D
        except AttributeError:
            pass

        return keras.models.load_model(model_path, custom_objects=custom_objects)

    @staticmethod
    def convert(model_path, output_dir, output_stripped_model_path=None):
        # cut out to_multi_gpu stuff (this could possibly break some models which don't use to_multi_gpu)
        model = KerasToTensorflow.load_keras_model(model_path)
        stripped_model = next((l for l in model.layers if isinstance(l, keras.engine.training.Model)), None)
        if stripped_model:
            if output_stripped_model_path is None:
                output_stripped_model_path = '%s-stripped%s' % os.path.splitext(model_path)
            stripped_model.save(output_stripped_model_path)
            model_path = output_stripped_model_path

        keras.backend.clear_session()
        session = tensorflow.Session()
        keras.backend.set_session(session)

        # disable loading of learning nodes
        keras.backend.set_learning_phase(0)

        model = KerasToTensorflow.load_keras_model(model_path)

        builder = tensorflow.saved_model.builder.SavedModelBuilder(output_dir)
        signature = tensorflow.saved_model.signature_def_utils.predict_signature_def(
            inputs={
                'image': model.input
            },
            outputs={
                'class_probabilities': model.output
            }
        )

        builder.add_meta_graph_and_variables(
            sess=keras.backend.get_session(),
            tags=[tensorflow.saved_model.tag_constants.SERVING],
            signature_def_map={
                tensorflow.saved_model.signature_constants.DEFAULT_SERVING_SIGNATURE_DEF_KEY: signature
            }
        )
        builder.save()



#!/usr/bin/env python

from setuptools import setup, find_packages


setup(
    name='model-converters',
    version='1.0.0',
    description='Tools for converting Keras models for use with other ML frameworks.',
    author='Triage Technologies Inc.',
    author_email='ai@triage.com',
    url='https://www.triage.com/',
    packages=find_packages(exclude=['tests', '.cache', '.venv', '.git', 'dist']),
    scripts=[
        'bin/keras_to_tensorflow',
    ],
    install_requires=[
        'tensorflow >= 0.12.0',
        'Keras >= 1.0.7',
        'h5py',
        'keras_model_specs',
        #'coremltools' <- NOTE!: coremltools is currently Python 2.7 only
    ]
)
